<?php

define('SUPPORT_PLAN_AGGREGATE', 0);
define('SUPPORT_PLAN_USER', 1);

define('SUPPORT_PLAN_THISWEEK', 0);
define('SUPPORT_PLAN_LASTWEEK', 1);
define('SUPPORT_PLAN_NEXTWEEK', 2);
define('SUPPORT_PLAN_THISMONTH', 5);
define('SUPPORT_PLAN_LASTMONTH', 6);
define('SUPPORT_PLAN_NEXTMONTH', 7);
define('SUPPORT_PLAN_THISQUARTER', 10);
define('SUPPORT_PLAN_LASTQUARTER', 11);
define('SUPPORT_PLAN_NEXTQUARTER', 12);
define('SUPPORT_PLAN_THISYEAR', 15);
define('SUPPORT_PLAN_LASTYEAR', 16);
define('SUPPORT_PLAN_NEXTYEAR', 17);
define('DIVIDER1', 9991);
define('DIVIDER2', 9992);
define('DIVIDER3', 9993);

function support_plan_admin_reports() {
  $form = array();

  // $period must be an integer
  $period = isset($_GET['rp']) ? (int)$_GET['rp'] : NULL;
  // $type must be an integer
  $type = isset($_GET['rt']) ? (int)$_GET['rt'] : NULL;
  // $roles must be a comma-separated list of integer
  $roles = isset($_GET['rr']) ? explode(',', preg_replace('/[^0-9,]/', '', $_GET['rr'])) : NULL;
  // $clients must be a comma-separated list of integer
  $clients = isset($_GET['rc']) ? explode(',', preg_replace('/[^0-9,]/', '', $_GET['rc'])) : NULL;

  if (!is_null($period) && !is_null($period)) {
    switch ($type) {
      case SUPPORT_PLAN_AGGREGATE:
      default:
        $report_type = 'aggregate';
        break;
      case SUPPORT_PLAN_USER:
        $report_type = 'user';
        break;
    }

    switch ($period) {
      case SUPPORT_PLAN_THISWEEK:
      default:
        $start = _support_plan_first_day(time());
        $end = $start + 86400 * 7;
        $days = 7;
        break;
      case SUPPORT_PLAN_LASTWEEK:
        $start = _support_plan_first_day(time() - 86400 * 7);
        $end = $start + 86400 * 7;
        $days = 7;
        break;
      case SUPPORT_PLAN_NEXTWEEK:
        $start = _support_plan_first_day(time() + 86400 * 7);
        $end = $start + 86400 * 7;
        $days = 7;
        break;
      case SUPPORT_PLAN_THISMONTH:
        $start = _support_plan_first_day_month(time());
        $end = _support_plan_last_day_month(time());
        $days = _support_plan_days_in_month(time());
        break;
      case SUPPORT_PLAN_LASTMONTH:
        $start = _support_plan_first_day_month(time() - 86400 * 31);
        $end = _support_plan_last_day_month(time() - 86400 * 31);
        $days = _support_plan_days_in_month(time() - 86400 * 31);
        break;
      case SUPPORT_PLAN_NEXTMONTH:
        $start = _support_plan_first_day_month(time() + 86400 * 31);
        $end = _support_plan_last_day_month(time() + 86400 * 31);
        $days = _support_plan_days_in_month(time() + 86400 * 31);
        break;
      case SUPPORT_PLAN_THISQUARTER:
        // TODO
        break;
      case SUPPORT_PLAN_LASTQUARTER:
        // TODO
        break;
      case SUPPORT_PLAN_NEXTQUARTER:
        // TODO
        break;
      case SUPPORT_PLAN_THISYEAR:
        // TODO
        break;
      case SUPPORT_PLAN_LASTYEAR:
        // TODO
        break;
      case SUPPORT_PLAN_NEXTYEAR:
        // TODO
        break;
    }

    $header_details = array('');
    $row = array('<strong>'. t('Plan') .'</strong>');
    $row2 = array('<strong>'. t('Actual') .'</strong>');
    $totals = array();
    $hours_client = array();
    $hours_user = array();
    for ($i = 0; $i < $days; $i++) {
      $date = $start + 86400 * $i;
      $header_details[] = t('!day<br />!date', array('!day' => format_date($date, 'custom', 'l'), '!date' => format_date($date, 'custom', 'M d')));
      $day = support_plan_day_load($date);
      $client_day = support_plan_aggregate_client($day);
      if (is_array($client_day)) {
        $row[] = theme('support_plan_user_client_hours_details', $client_day, 64);
      }
      else {
        $row[] = '';
      }
      // Add up totals
      if (is_array($client_day)) {
        foreach ($client_day as $clid => $data) {
          $totals['client_plan'][$clid] += $data->hours;
        }
      }
      else if (!is_array($totals['client_plan'])) {
        $totals['client_plan'] = array();
      }

      $user_day = support_plan_aggregate_user($day);
      if (is_array($user_day)) {
        $row_user[] = theme('support_plan_user_client_hours_details', $user_day, 64);
      }
      else {
        $row_user[] = '';
      }
      // Add up totals
      if (is_array($user_day)) {
        foreach ($user_day as $uid => $data) {
          $totals['user_plan'][$uid] += $data->hours;
        }
      }
      else if (!is_array($totals['user_plan'])) {
        $totals['user_plan'] = array();
      }

      // Integrate with the support_timer module, if enabled
      if (module_exists('support_timer')) {
        $hour_client = array();
        $hour_user = array();
        // The support_timer module uses a slightly different date format
        $convert = strtotime(date('d M Y', $date));
        $result = db_query('SELECT tt.time, t.client, n.uid FROM {support_ticket_timer} tt LEFT JOIN {support_ticket} t ON tt.nid = t.nid LEFT JOIN {node} n ON t.nid = n.nid WHERE tt.date = %d', $convert);
        while ($timer = db_fetch_object($result)) {
          $hour_client[$timer->client]->hours += support_plan_timer_to_hours($timer->time);
          $hours_client[$timer->client]->hours += support_plan_timer_to_hours($timer->time);
          $hour_user[$timer->uid]->hours += support_plan_timer_to_hours($timer->time);
          $hours_user[$timer->uid]->hours += support_plan_timer_to_hours($timer->time);
        }
        $result = db_query('SELECT tt.time, t.client, c.uid FROM {support_ticket_comment_timer} tt LEFT JOIN {support_ticket_comment} t ON tt.cid = t.cid LEFT JOIN {comments} c ON t.cid = c.cid WHERE tt.date = %d', $convert);
        while ($timer = db_fetch_object($result)) {
          $hour_client[$timer->client]->hours += support_plan_timer_to_hours($timer->time);
          $hours_client[$timer->client]->hours += support_plan_timer_to_hours($timer->time);
          $hour_user[$timer->uid]->hours += support_plan_timer_to_hours($timer->time);
          $hours_user[$timer->uid]->hours += support_plan_timer_to_hours($timer->time);
        }
        $row2[] = theme('support_plan_user_client_hours_details', $hour_client, 64);
        $row_user[] = theme('support_plan_user_client_hours_details', $hour_user, 64);
      }
    }
    $rows_details = array($row);
    // Only display actual data if support_timer is enabled to collect it
    if (count($row2) > 1) {
      $rows_details[] = $row2;
      foreach ($hours_client as $clid => $data) {
        // Add up totals
        $totals['client_actual'][$clid] = $data->hours;
      }
    }
    if (count($row_user) > 1) {
      foreach ($hours_user as $uid => $data) {
        // Add up totals
        $totals['user_actual'][$uid] = $data->hours;
      }
    }

    $header_client = array(t('Plan'));
    $client_plan_sum = is_array($totals['client_plan']) ? array_sum($totals['client_plan']) : 0;
    $client_actual_sum = is_array($totals['client_actual']) ? array_sum($totals['client_actual']) : 0;
    $max = $client_plan_sum > $client_actual_sum ? $client_plan_sum : $client_actual_sum;
    $row = array(theme('support_plan_user_hours_summary', $totals['client_plan'], 'support_client_load', $max, t('Not scheduled')));
    if (count($row2) > 1) {
      $header_client[] = t('Actual');
      $row[] = theme('support_plan_user_hours_summary', $totals['client_actual'], 'support_client_load', $max, t('Not worked'));
    }
    $rows_client = array($row);

    $header_user = array(t('Plan'));
    $user_plan_sum = is_array($totals['user_plan']) ?  array_sum($totals['user_plan']) : 0;
    $user_actual_sum = is_array($totals['user_actual']) ? array_sum($totals['user_actual']) : 0;
    $max = $user_plan_sum > $user_actual_sum ? $user_plan_sum : $user_actual_sum;
    $row = array(theme('support_plan_user_hours_summary', $totals['user_plan'], 'user_load', $max, t('Not scheduled')));
    if (count($row2) > 1) {
      $header_user[] = t('Actual');
      $row[] = theme('support_plan_user_hours_summary', $totals['user_actual'], 'user_load', $max, t('Not worked'));
    }
    $rows_user = array($row);

    $form['client'] = array(
      '#type' => 'fieldset',
      '#title' => t('Client'),
    );
    $form['client']['client_data'] = array(
      '#type' => 'markup',
      '#value' => theme('table', $header_client, $rows_client, array('id' => 'support_plan_summary')),
    );

    $form['user'] = array(
      '#type' => 'fieldset',
      '#title' => t('User'),
    );
    $form['user']['user_data'] = array(
      '#type' => 'markup',
      '#value' => theme('table', $header_user, $rows_user, array('id' => 'support_plan_summary')),
    );

    $form['detail'] = array(
      '#type' => 'fieldset',
      '#title' => t('Detail'),
    );
    $form['detail']['detail_data'] = array(
      '#type' => 'markup',
      '#value' => theme('table', $header_details, $rows_details, array('id' => 'support_plan_week')),
    );
  }

  // TODO: Custom periods
  $form['report_period'] = array(
    '#type' => 'select',
    '#title' => t('Period'),
    //'#options' => array(SUPPORT_PLAN_THISWEEK => t('This week'), SUPPORT_PLAN_LASTWEEK => t('Last week'), SUPPORT_PLAN_NEXTWEEK => t('Next week'), DIVIDER1 => '--', SUPPORT_PLAN_THISMONTH => t('This month'), SUPPORT_PLAN_LASTMONTH => t('Last month'), SUPPORT_PLAN_NEXTMONTH => t('Next month'), DIVIDER2 => '--', SUPPORT_PLAN_THISQUARTER => t('This quarter'), SUPPORT_PLAN_LASTQUARTER => t('Last quarter'), SUPPORT_PLAN_NEXTQUARTER => t('Next quarter'), DIVIDER3 => '--', SUPPORT_PLAN_THISYEAR => t('This year'), SUPPORT_PLAN_LASTYEAR => t('Last year'), SUPPORT_PLAN_NEXTYEAR => t('Next year')),
    '#options' => array(SUPPORT_PLAN_THISWEEK => t('This week'), SUPPORT_PLAN_LASTWEEK => t('Last week'), SUPPORT_PLAN_NEXTWEEK => t('Next week'), DIVIDER1 => '--', SUPPORT_PLAN_THISMONTH => t('This month'), SUPPORT_PLAN_LASTMONTH => t('Last month'), SUPPORT_PLAN_NEXTMONTH => t('Next month')),
    '#default_value' => !is_null($period) ? $period : 0,
    '#description' => t('Select the duration of your report.'),
    '#required' => TRUE,
  );

  $form['report_type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    //'#options' => array(SUPPORT_PLAN_AGGREGATE => t('Aggregate'), SUPPORT_PLAN_USER => t('Per-user')),
    '#options' => array(SUPPORT_PLAN_AGGREGATE => t('Aggregate')),
    '#default_value' => !is_null($type) ? $type : 0,
    '#description' => t("Select the type of report to generate."),
    '#required' => TRUE,
  );

  $form['optional'] = array(
    '#type' => 'fieldset',
    '#title' => t('Optional'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['optional']['report_role'] = array(
    '#type' => 'select',
    '#title' => t('Role(s)'),
    '#multiple' => TRUE,
    '#options' => user_roles(),
    '#description' => t('Optionally limit your report to one or more roles.'),
  );

  $form['optional']['report_client'] = array(
    '#type' => 'select',
    '#title' => t('Client(s)'),
    '#multiple' => TRUE,
    '#options' => _support_available_clients(),
    '#description' => t('Optionally limit your report to one or more clients.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Generate report'),
  );

  return $form;
}

function support_plan_admin_reports_submit($form, &$form_state) {
  $args = array();
  $args['rp'] = $form_state['values']['report_period'];
  $args['rt'] = $form_state['values']['report_type'];
  if (!empty($form_state['values']['report_role'])) {
    $args['rr'] = implode(',', $form_state['values']['report_role']);
  }
  if (!empty($form_state['values']['report_client'])) {
    $args['rc'] = implode(',', $form_state['values']['report_client']);
  }
  drupal_goto('admin/support/plan_reports', $args);
}

function _support_plan_first_day_month($time) {
  return (strtotime(date('M 1, Y 12:00', $time)));
}

function _support_plan_last_day_month($time) {
  return (strtotime(date('M '. _support_plan_days_in_month($time) .', Y 12:00', $time)));
}

function _support_plan_days_in_month($time) {
  return date('j', (mktime(date('H', $time), date('i', $time), date('s', $time), date('n', $time), -1)));
}

function support_plan_aggregate_client($day) {
  // TODO: Count the # of people involved, give 12 hours per person to breakdown scale
  $aggregate = array();
  foreach ($day as $uid => $client) {
    foreach ($client as $clid => $data) {
      if (isset($aggregate[$clid])) {
        $aggregate[$clid]->hours += $data->hours;
      }
      else {
        $aggregate[$clid] = drupal_clone($data);
      }
    }
  }
  return $aggregate;
}

function support_plan_aggregate_user($day) {
  // TODO: Count the # of people involved, give 12 hours per person to breakdown scale
  $aggregate = array();
  foreach ($day as $uid => $client) {
    foreach ($client as $clid => $data) {
      if (isset($aggregate[$uid])) {
        $aggregate[$uid]->hours += $data->hours;
      }
      else {
        $aggregate[$uid] = drupal_clone($data);
      }
    }
  }
  return $aggregate;
}
